<?php
/**
 * Plugin MailShot
 * (c) 2012 Cedric Morin
 * Licence GNU/GPL
 */

if (!defined('_ECRIRE_INC_VERSION')){
	return;
}
include_spip("inc/config");
include_spip("classes/facteur");


/**
 * @param array $to_send
 *   string email
 *   string sujet
 *   string html
 *   string texte
 * @param array $options
 *   bool filtre_images
 *   array smtp
 *     string host
 *     string port
 *     string auth
 *     string username
 *     string password
 *     string secure
 *     string errorsto
 *   string adresse_envoi_nom
 *   string adresse_envoi_email
 * @return Facteur
 */
function &bulkmailer_sendinblue_dist($to_send, $options = array()){
	static $config = null;
	static $mailer_defaut;
	if (is_null($config)){
		$config = lire_config("mailshot/");
		$mailer_defaut = charger_fonction("defaut", "bulkmailer");
	}

	// on utilise une surcharge pour utiliser l'API http
	$options['sender_class'] = "FacteurSendinblue";
	return $mailer_defaut($to_send, $options);

}


/**
 * Configurer sendinblue : declarer le sender si besoin
 * appele depuis traiter() de formulaire_configurer_mailshot
 * @param $res
 */
function bulkmailer_sendinblue_config_dist(&$res){
	$sender_mail = "";

	include_spip('inc/config');
	$config = lire_config('mailshot/');
	if ($config['adresse_envoi']=='oui'){
		$sender_mail = $config['adresse_envoi_email'];
		$sender_name = $config['adresse_envoi_nom'];
	} else {
		include_spip("classes/facteur");
		$facteur = new Facteur("example@example.org", "", "", "");
		$sender_mail = $facteur->From;
		$sender_name = $facteur->FromName;
	}

	// si le sender n'est pas dans les emails de sendinblue l'ajouter
	if ($sender_mail){
		sendinblue_add_sender($sender_mail, $sender_name, true);
	}

	// debug purpose : decommenter pour tester l'init en validant le formulaire de configuration de mailshot
	// bulkmailer_sendinblue_init_dist(0);

}

function sendinblue_api_config(){
	static $sib_config;
	if (is_null($sib_config)){
		include_spip('inc/config');
		$config = lire_config('mailshot/');
		include_spip('lib/sendinblue-apiv3-php/autoloader');

		// Configure API key authorization: api-key
		$sib_config = SendinBlue\Client\Configuration::getDefaultConfiguration()->setApiKey('api-key', $config['sendinblue_api_key']);
		// Uncomment below to setup prefix (e.g. Bearer) for API key, if needed
		// $config = SendinBlue\Client\Configuration::getDefaultConfiguration()->setApiKeyPrefix('api-key', 'Bearer');

		// Configure API key authorization: partner-key
		// $config = SendinBlue\Client\Configuration::getDefaultConfiguration()->setApiKey('partner-key', 'YOUR_API_KEY');

		// Uncomment below to setup prefix (e.g. Bearer) for API key, if needed
		// $config = SendinBlue\Client\Configuration::getDefaultConfiguration()->setApiKeyPrefix('partner-key', 'Bearer');
	}

	return $sib_config;
}

/**
 * Verifier si un email d'envoi est dans la liste des senders sendinblue
 * et sinon l'ajoute
 *
 * @param string $sender_email
 * @param string $sender_name
 * @param bool $force
 * @return bool
 */
function sendinblue_add_sender($sender_email, $sender_name, $force = false){

	$config = sendinblue_api_config();
	$apiInstance = new SendinBlue\Client\Api\SendersApi(null, $config);
	try {
		$domain = explode('@', $sender_email);
		$domain = end($domain);
		$senders = $apiInstance->getSenders('', $domain);
	} catch (Exception $e) {
		spip_log("Exception when calling SendersApi->getSenders : " . $e->getMessage(), 'mailshot' . _LOG_ERREUR);
		$senders = null;
	}

	if ($senders){
		#spip_log(var_export($senders->getSenders(), true), 'dbgmailshot');
		foreach ($senders->getSenders() as $sender){
			if ($sender->getEmail()===$sender_email and $sender->getName()===$sender_name){
				if (!$sender->getActive() and $force){
					// on le supprime pour le recreer et relancer un email de confirmation
					spip_log("sendinblue_add_sender: suppression sender $sender_email | $sender_name pour le recreer", 'mailshot');
					try {
						$apiInstance->deleteSender($sender->getId());
					} catch (Exception $e) {
						spip_log("Exception when calling SendersApi->deleteSender : " . $e->getMessage(), 'mailshot' . _LOG_ERREUR);
					}
				} else {
					// rien a faire
					spip_log("sendinblue_add_sender: $sender_email | $sender_name deja a jour, rien a faire", 'mailshot');
					return true;
				}
			}
		}
	}

	spip_log("sendinblue_add_sender: $sender_email n'est pas un sender connu, on l'ajoute", 'mailshot');

	$sender = new \SendinBlue\Client\Model\CreateSender(['email' => $sender_email, 'name' => $sender_name]);
	try {
		$result = $apiInstance->createSender($sender);
	} catch (Exception $e) {
		spip_log("Exception when calling SendersApi->createSender : " . $e->getMessage(), 'mailshot' . _LOG_ERREUR);
		$result = null;
	}

	return true;
}



/**
 * Initialiser sendinblue : declarer un eventcallbackurl pour recuperer les retours sur bounce, reject, open, clic....
 *
 * @param int $id_mailshot
 * @return bool
 */
function bulkmailer_sendinblue_init_dist($id_mailshot=0){

	spip_log("bulkmailer_sendinblue_init_dist $id_mailshot","mailshot");

	// si il y a un envoyeur specifique, verifier qu'il est dans les senders
	if ($id_mailshot
		and $mailshot = sql_fetsel('*', 'spip_mailshots', 'id_mailshot=' . intval($id_mailshot))) {
		if (!empty($mailshot['from_email'])) {
			sendinblue_add_sender($mailshot['from_email'], $mailshot['from_name'], true);
		}
	}

	// le webhook dont on a besoin
	$webhook_url = url_absolue(_DIR_RACINE."mailshot_webhook.api/sendinblue/");
	$webhook_events = [
		"opened",
		"uniqueOpened", // a priori redondant de opened pour nous, a confirmer ?
		"click",
		"softBounce",
		"hardBounce",
		"spam",
		"invalid",
		"blocked",
		//"error", // generer une erreur quand on essaye de mettre cet event ET si on le met depuis l'interface on a null a la place
		"unsubscribed",
	];


	// lister les webhooks existants
	$config = sendinblue_api_config();
	$apiInstance = new SendinBlue\Client\Api\WebhooksApi(null, $config);
	$type = 'transactional';

	try {
		$result = $apiInstance->getWebhooks($type);
		//spip_log(var_export($result, true), 'dbgmailshot'. _LOG_DEBUG);
	} catch (Exception $e) {
		spip_log("bulkmailer_sendinblue_init_dist: Exception when calling WebhooksApi->getWebhooks : " . $e->getMessage(), 'mailshot' . _LOG_ERREUR);
		$result = null;
	}

	$deja_events = [];
	if ($result) {
		$webhooks = $result->getWebhooks();
		//spip_log(var_export($webhooks, true), 'dbgmailshot'. _LOG_DEBUG);
		foreach ($webhooks as $webhook) {
			if ($webhook['url'] === $webhook_url) {
				$webhook_deja = $webhook;
			}
			else {
				// supprimer les autres webhooks car ils vont recevoir des feedbacks de notre emailing
				try {
					$apiInstance->deleteWebhook($webhook['id']);
					spip_log("bulkmailer_sendinblue_init_dist: WebhooksApi->deleteWebhook #" .$webhook['id'], 'mailshot' . _LOG_DEBUG);
				} catch (Exception $e) {
					spip_log("bulkmailer_sendinblue_init_dist: Exception when calling WebhooksApi->deleteWebhook : " . $e->getMessage(), 'mailshot' . _LOG_ERREUR);
					$result = null;
				}
			}
		}
	}

	$webhook_todo = [
		'url' => $webhook_url,
		'description' => 'Webhook auto mailshot/SPIP',
		'events' => $webhook_events,
		'type' => 'transactional'
	];

	if ($webhook_deja) {
		$webhookId = $webhook_deja['id'];
		$missing_events = array_diff($webhook_events, $webhook_deja['events']);
		if ($missing_events){
			spip_log("Missing events : " .implode(', ', $missing_events), 'dbgmailshot');
			$webhook_todo['events'] = array_merge($webhook_todo['events'], $deja_events);
			$webhook_todo['events'] = array_filter($webhook_todo['events']);

			$updateWebhook = new \SendinBlue\Client\Model\UpdateWebhook($webhook_todo); // \SendinBlue\Client\Model\UpdateWebhook | Values to update a webhook

			try {
				$apiInstance->updateWebhook($webhookId, $updateWebhook);
				spip_log("bulkmailer_sendinblue_init_dist: WebhooksApi->updateWebhook #$webhookId", 'mailshot' . _LOG_DEBUG);
			} catch (Exception $e) {
				spip_log("bulkmailer_sendinblue_init_dist: Exception when calling WebhooksApi->updateWebhook : " . $e->getMessage(), 'mailshot' . _LOG_ERREUR);
				$result = null;
			}
		}
		else {
			spip_log("bulkmailer_sendinblue_init_dist: Rien a faire : webhook #$webhookId complet", 'mailshot' . _LOG_DEBUG);
		}
	}
	else {
		$createWebhook = new \SendinBlue\Client\Model\CreateWebhook($webhook_todo); // \SendinBlue\Client\Model\CreateWebhook | Values to create a webhook

		try {
		    $result = $apiInstance->createWebhook($createWebhook);
		    spip_log("bulkmailer_sendinblue_init_dist: WebhooksApi->createWebhook : " . json_encode($webhook_todo), 'mailshot' . _LOG_DEBUG);
		} catch (Exception $e) {
			spip_log("bulkmailer_sendinblue_init_dist: Exception when calling WebhooksApi->createWebhook : " . $e->getMessage(), 'mailshot' . _LOG_ERREUR);
			$result = null;
		}
	}

	return true;

}


/**
 * Prendre en charge le webhook sendinblue
 *
 * @param $arg
 */
function bulkmailer_sendinblue_webhook_dist($arg){

	if ($_SERVER['REQUEST_METHOD'] == 'HEAD'){
		http_status(200);
		exit;
	}

	$data = file_get_contents('php://input');
	spip_log("bulkmailer_sendinblue_webhook_dist $data","mailshot_feedback");

	if (!$data OR !$events = json_decode($data, true)){
		http_status(403);
		exit;
	}

	// si un seul event, on le met dans un tableau pour tout traiter de la meme facon
	if (isset($events['event']) and isset($events['message-id'])){
		$events = [$events];
	}

	foreach($events as $event){
		// [
		// "request", "deferred", "delivered", "opened", "unique_opened", "click",
		// "soft_bounce", "hard_bounce", "complaint", "invalid_email", "blocked", "error", "unsubscribed"
		// ];

		$quoi = $event['event'];
		if (in_array($quoi, ["request", "deferred", "delivered"])) {
			continue; // on ignore ces events qu'on a en principe pas demande
		}
		switch ($quoi) {
			case 'unique_opened':
			case 'opened':
				$quoi = 'read';
				break;
			case 'click':
				$quoi = 'clic';
				break;
			case 'blocked':
			case 'error':
			case 'invalid_email':
				$quoi = 'reject';
				break;
			case 'complaint':
				$quoi = 'spam';
				break;
			case 'unsubscribed':
				$quoi = 'unsub';
				break;
		}

		$email = $event['email'];
		if (!empty($event['tags'])) {
			foreach ($event['tags'] as $tracking_id) {
				if (strpos($tracking_id, '/#') !==false) {
					$tracking_id = explode('/#',$tracking_id);
					if (reset($tracking_id)==protocole_implicite($GLOBALS['meta']['adresse_site'])){
						$tracking_id = end($tracking_id);
						spip_log("tracking $quoi $email $tracking_id",'mailshot_feedback');
						// appeler l'api webhook mailshot avec le raw_feedback en plus pour la trace en base
						$feedback = charger_fonction("feedback","newsletter");
						$feedback("$quoi|".$event['event'],$email,$tracking_id);
					}
				}
			}
		}
	}

}


/**
 * Class FacteurSendinblue
 */
class FacteurSendinblue extends Facteur {

	public $send_options = array();
	protected $recipients = [
		'to' => [/*['email' => '','name' => '']*/],
		'cc' => [/*['email' => '','name' => '']*/],
		'bcc' => [/*['email' => '','name' => '']*/],
		'reply-to' => [/*['email' => '','name' => '']*/],
	];
	protected $headers = [

	];

	protected function cleanAdress($address, $name = ''){
		$address = trim($address);
		$name = trim(preg_replace('/[\r\n]+/', '', $name)); //Strip breaks and trim
		if (!self::ValidateAddress($address)){
			$this->SetError('invalid_address' . ': ' . $address);
			return false;
		}
		return array($address, $name);
	}

	/**
	 * Adds a "To" address.
	 * @param string $address
	 * @param string $name
	 * @return boolean true on success, false if address already used
	 */
	public function AddAddress($address, $name = '', $index = 'to'){
		if ($a = $this->cleanAdress($address, $name)){
			$this->recipients[$index][] = array('email' => $a[0], 'name' => $a[1]);
			return true;
		}
		return false;
	}

	/**
	 * Adds a "Cc" address.
	 * Note: this function works with the SMTP mailer on win32, not with the "mail" mailer.
	 * @param string $address
	 * @param string $name
	 * @return boolean true on success, false if address already used
	 */
	public function AddCC($address, $name = ''){
		return $this->AddAddress($address, $name, 'cc');
	}

	/**
	 * Adds a "Bcc" address.
	 * Note: this function works with the SMTP mailer on win32, not with the "mail" mailer.
	 * @param string $address
	 * @param string $name
	 * @return boolean true on success, false if address already used
	 */
	public function AddBCC($address, $name = ''){
		return $this->AddAddress($address, $name, 'bcc');
	}

	/**
	 * Adds a "Reply-to" address.
	 * @param string $address
	 * @param string $name
	 * @return boolean
	 */
	public function AddReplyTo($address, $name = ''){
		return $this->AddAddress($address, $name, 'reply-to');
	}

	/**
	 * Adds a custom header.
	 * @access public
	 * @return void
	 */
	public function AddCustomHeader($name, $value = null){
		if ($value===null){
			// Value passed in as name:value
			list($name, $value) = explode(':', $name, 2);
		}
		$this->headers[$name] = trim($value);
	}

	/**
	 * utilise $this->send_options options d'envoi
	 *     string tracking_id
	 * @return bool
	 */
	public function Send(){
		$config = sendinblue_api_config();

		$data = [
			'sender' => new \SendinBlue\Client\Model\SendSmtpEmailSender(['email' => $this->From, 'name' => $this->FromName]),
			'to' => [],
			'bcc' => [],
			'cc' => [],
			'htmlContent' => $this->Body,
			'textContent' => $this->AltBody,
			'subject' => $this->Subject,
			'tags' => [],
		];
		foreach ($this->recipients['to'] as $to){
			$data['to'][] = new \SendinBlue\Client\Model\SendSmtpEmailTo(array_filter($to));
		}
		foreach ($this->recipients['bcc'] as $bcc){
			$data['bcc'][] = new \SendinBlue\Client\Model\SendSmtpEmailBcc(array_filter($bcc));
		}
		foreach ($this->recipients['cc'] as $cc){
			$data['cc'][] = new \SendinBlue\Client\Model\SendSmtpEmailCc(array_filter($cc));
		}
		if ($this->recipients['reply-to']
		  and $replyto = end($this->recipients['reply-to'])){
			$data['replyTo'] = new \SendinBlue\Client\Model\SendSmtpEmailReplyTo(array_filter($replyto));
		}

		// TODO ?
		//$this->container['attachment'] = isset($data['attachment']) ? $data['attachment'] : null;

		if (!empty($this->headers)){
			$data['headers'] = $this->headers;
		}

		// ajouter le tracking_id en tag, pour retrouver le message apres webhook
		if (isset($this->send_options['tracking_id'])
			and $id = $this->send_options['tracking_id']){
			// prefixer le tracking par l'url du site pour ne pas melanger les feedbacks
			$data['tags'][] = protocole_implicite($GLOBALS['meta']['adresse_site']) . "/#" . $this->send_options['tracking_id'];
		}

		// ne pas envoyer de valeurs vides pour ces cles
		foreach (['to', 'bcc', 'cc', 'tags'] as $k) {
			if (empty($data[$k])) {
				unset($data[$k]);
			}
		}
		$sendSmtpEmail = new \SendinBlue\Client\Model\SendSmtpEmail($data);

		$apiInstance = new SendinBlue\Client\Api\TransactionalEmailsApi(null, $config);

		try {
			$result = $apiInstance->sendTransacEmail($sendSmtpEmail);
			spip_log("FacteurSendinblue->send() : result " . json_encode($result), 'mailshot' . _LOG_DEBUG);
			return true;

		} catch (Exception $e) {
			spip_log("Exception when calling TransactionalEmailsApi->sendTransacEmail : " . $e->getMessage(), 'mailshot' . _LOG_ERREUR);
			$this->SetError("status " .$e->getMessage());
			return false;
		}

	}

	public function CreateHeader(){
	}
}
